package com.ims.eval.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.config.ImaConfig;
import com.ims.eval.entity.EvaluationNotice;
import com.ims.eval.entity.OrganizationStructure;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.dto.result.ResultInfo;
import com.ims.eval.service.IEvaluationNoticeService;
import com.ims.eval.service.IOrganizationStructureService;
import com.ims.eval.util.FileUploadUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.*;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

/**
 * 通告管理
 *
 * @author hlf
 * @date 2023/5/17 14:48
 * 文件说明：
 */
@RestController
@RequestMapping("//evaluation-dept-notice")
public class NoticeManagementController {

	@Autowired
	private IEvaluationNoticeService evaluationNoticeService;

	@Autowired
	private IOrganizationStructureService organizationStructureService;

	@Autowired
	HttpServletRequest request;

	/**
	 * 通告管理列表信息（分页）
	 *
	 * @param pageNum     当前记录起始索引
	 * @param pageSize    每页显示记录数
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "noticeTitle", required = false) String noticeTitle) {
		IPage<EvaluationNotice> list = evaluationNoticeService.listPage(pageNum, pageSize, noticeTitle);
		List<EvaluationNotice> evaluationNoticeList = list.getRecords();
		for (EvaluationNotice obj : evaluationNoticeList) {
			StringBuilder names = new StringBuilder();
			if (!"".equals(obj.getSendToContent()) && null != obj.getSendToContent()) {
				String[] strings = obj.getSendToContent().split(",");
				for (String str : strings) {
					List<OrganizationStructure> deptList = organizationStructureService.getTree("23031001", 1, "140");
					if (deptList.size() > 0) {
						OrganizationStructure organizationStructure = deptList.stream().filter(item -> item.getId().equals(str)).findFirst().orElse(null);
						if (ObjectUtil.isNotNull(organizationStructure)) {
							names.append(organizationStructure.getName()).append(",");
						}
					}
				}
				obj.setDeptNames(names.toString().substring(0, names.toString().length() - 1));
			}
		}
		return R.ok().data(list);
	}

	/**
	 * 通告管理信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationNotice> list = evaluationNoticeService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增/修改通告管理信息
	 *
	 * @param evaluationNotice 通告管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationNotice evaluationNotice) {
		try {
			String id;
			if (StringUtils.isEmpty(evaluationNotice.getId())) {//true：为空
				id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
				evaluationNotice.setId(id);
				evaluationNotice.setCreateTime(new Date());
			} else {
				id = evaluationNotice.getId();
				evaluationNotice.setUpdateTime(new Date());
			}
			boolean b = evaluationNoticeService.saveOrUpdate(evaluationNotice);
			if (b) {
				return R.ok().data(id);
			} else {
				return R.error("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除通告管理信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = evaluationNoticeService.removeByIds(Arrays.asList(strings), request);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 修改状态
	 *
	 * @param ids          多主键
	 * @param releaseState 状态（未发布或已发布或草稿）
	 * @return 结果
	 */
	@PostMapping(value = "/modifiedState")
	public R modifiedState(@RequestParam(value = "ids") String ids,
						   @RequestParam(value = "releaseState") String releaseState) {
		try {
			boolean b = false;
			if (ids.length() > 0) {
				String[] strings = ids.split(",");
				for (String id : strings) {
					EvaluationNotice evaluationNotice = new EvaluationNotice();
					evaluationNotice.setId(id);
					if ("已发布".equals(releaseState)) {
						evaluationNotice.setReleaseDate(new Date());
					} else if ("未发布".equals(releaseState)) {
						evaluationNotice.setReleaseDate(null);
					}
					evaluationNotice.setReleaseState(releaseState);
					b = evaluationNoticeService.updateById(evaluationNotice);
				}
			}
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 上传附件
	 *
	 * @param id    主键
	 * @param files 附件集
	 * @return 结果
	 * @throws IOException 异常
	 */
	@PostMapping(value = "/uploadAttachment")
	public R uploadAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "files", required = false) MultipartFile[] files) {
		try {
			boolean b = evaluationNoticeService.uploadAttachment(id, files, request);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("上传附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 删除附件
	 *
	 * @param id      主键
	 * @param docPath 附件路径
	 * @return 结果
	 */
	@PostMapping(value = "/deleteAttachment")
	public R deleteAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "docPath") String docPath) {
		try {
			boolean b = evaluationNoticeService.deleteAttachment(id, docPath, request);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 通告展示
	 *
	 * @param deptId      部门主键
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/circularDisplay")
	public R circularDisplay(@RequestParam(value = "deptId") String deptId,
							 @RequestParam(value = "noticeTitle") String noticeTitle,
							 @RequestParam(value = "beginDataTime", required = false) String beginDataTime,
							 @RequestParam(value = "endDataTime", required = false) String endDataTime) {
		List<EvaluationNotice> list = evaluationNoticeService.circularDisplay(deptId, noticeTitle, beginDataTime, endDataTime);
		return R.ok().data(list);
	}
}
